import struct

import matplotlib.pyplot as plt
import numpy as np

#-----------------------------------------------------------------------------------------------------------------------------
#
# DATA FORMAT
#-------------
#
# format bin file with pixel positions
#
# imodule (int16_t), itile (int16_t), icrystal (int16_t), x (float), y (float), z (float)
#
# imodule -> 0 to 7
# itile -> 0 to 14
# icrystal -> 0 to 1633
# x -> -0.21 to 0.21
# y -> -0.21 to 0.21
# z -> -0.12 to 0.12 
#
# 1634*8*15 = 196080
#
#
#----------------------------------------------------------------------------|
#                                                                            |
# archivo de correspondencia entre el formato Debora y el Christoph:         |
#  lut_block_tile_module_columns.txt                                         |
#                                                                            |
#----------------------------------------------------------------------------|
#
#-------------------------------------------------------------------------------------------------------------------------------

# tuple_format = 'h h h f f f'
tuple_format = '<3h 3f'
tuple_size = struct.calcsize(tuple_format)


parsed_data = []

parsed_data1 = []
parsed_data2 = []
parsed_data3 = []
parsed_data4 = []
parsed_data5 = []
parsed_data6 = []
parsed_data7 = []

###################################################################################################
#
# Correspondencia [bloques Debora] con [blocks and tiles] de Christoph
#
#
####################################################################################################

#blockID,moduleID,tileID

corr = np.array([(0,0,0),(1,0,7),(2,0,11),(3,6,0),(4,6,7),(5,6,11),(6,2,0),(7,2,7),(8,2,11),(9,7,0),(10,7,7),(11,7,11),(12,1,0),(13,1,7),(14,1,11),(15,5,0),(16,5,7),
(17,5,11),(18,4,0),(19,4,7),(20,4,11),(21,3,0),(22,3,7),(23,3,11),(24,0,3),(25,0,9),(26,0,13),(27,6,3),(28,6,9),(29,6,13),(30,2,3),(31,2,9),(32,2,13),(33,7,3),(34,7,9),
(35,7,13),(36,1,3),(37,1,9),(38,1,13),(39,5,3),(40,5,9),(41,5,13),(42,4,3),(43,4,9),(44,4,13),(45,3,3),(46,3,9),(47,3,13),(48,0,1),(49,0,5),(50,0,10),(51,6,1),(52,6,5),
(53,6,10),(54,2,1),(55,2,5),(56,2,10),(57,7,1),(58,7,5),(59,7,10),(60,1,1),(61,1,5),(62,1,10),(63,5,1),(64,5,5),(65,5,10),(66,4,1),(67,4,5),(68,4,10),(69,3,1),(70,3,5),
(71,3,10),(72,0,4),(73,0,8),(74,0,14),(75,6,4),(76,6,8),(77,6,14),(78,2,4),(79,2,8),(80,2,14),(81,7,4),(82,7,8),(83,7,14),(84,1,4),(85,1,8),(86,1,14),(87,5,4),(88,5,8),
(89,5,14),(90,4,4),(91,4,8),(92,4,14),(93,3,4),(94,3,8),(95,3,14),(96,0,2),(97,0,6),(98,0,12),(99,6,2),(100,6,6),(101,6,12),(102,2,2),(103,2,6),(104,2,12),(105,7,2),
(106,7,6),(107,7,12),(108,1,2),(109,1,6),(110,1,12),(111,5,2),(112,5,6),(113,5,12),(114,4,2),(115,4,6),(116,4,12),(117,3,2),(118,3,6),(119,3,12)])

####################################################################################################
#
#   imodule, itile, icrystal, x, y, z 
#
with open("BP7_crystal_coord.bin", "rb") as f:
    # Step 1: Read the file format flag
    
    # Step 2: Read tuples until the end of the file
    while chunk := f.read(tuple_size):
        # Unpack each chunk into (a, b, c, d)
        a, b, c, d, e, g = struct.unpack(tuple_format, chunk)
        parsed_data.append((a, b, c, d, e, g))


        #aqui
        for i in range(len(corr)):
            if corr[:, 1][i] == a and corr[:, 2][i] == b:
                blockDeb = corr[:, 0][i]
                #print("El vector: ", (blockDeb, corr[:, 1][i], corr[:, 2][i]))
        
        if (blockDeb < 72 and blockDeb > 47):   # Un único anillo en el centro, en vez de 5 anillos
            # print(parsed_data1,parsed_data2,parsed_data3,parsed_data4,parsed_data5,parsed_data6)
            parsed_data1.append(a)
            parsed_data2.append(b)
            parsed_data3.append(c)
            parsed_data4.append(d)
            parsed_data5.append(e)
            parsed_data6.append(g)
            parsed_data7.append(blockDeb)

#        if len(parsed_data) >= 1:
#    	    break

print(tuple_size)
print("iMODULE -> min(parsed_data1): ", min(parsed_data1), "   max(parsed_data1): ", max(parsed_data1))
print("iTILE -> min(parsed_data2): ", min(parsed_data2), "   max(parsed_data2): ", max(parsed_data2))
print("iCRYSTAL -> min(parsed_data3): ", min(parsed_data3), "   max(parsed_data3): ", max(parsed_data3))
print("X -> min(parsed_data4): ", min(parsed_data4), "   max(parsed_data4): ", max(parsed_data4))
print("Y -> min(parsed_data5): ", min(parsed_data5), "   max(parsed_data5): ", max(parsed_data5))
print("Z -> min(parsed_data6): ", min(parsed_data6), "   max(parsed_data6): ", max(parsed_data6))
print("blockDeb -> min(parsed_data7): ", min(parsed_data7), "   max(parsed_data7): ", max(parsed_data7))



 
# Plotting a basic histogram
plt.hist(parsed_data1, color='skyblue', edgecolor='black')
# Adding labels and title
plt.xlabel('iMODULE')
plt.ylabel('Frequency')
plt.title('Histogram iMODULE')
# Display the plot
plt.show()



# Plotting a basic histogram
#plt.hist(parsed_data2, range=[0,119], bins=120, color='skyblue', edgecolor='black')
plt.hist(parsed_data2, color='skyblue', edgecolor='black') 
# Adding labels and title
plt.xlabel('iTILE')
plt.ylabel('Frequency')
plt.title('Histogram iTILE')
# Display the plot
plt.show()

# Plotting a basic histogram
plt.hist(parsed_data7, color='skyblue', edgecolor='black') 
# Adding labels and title
plt.xlabel('blockDeb')
plt.ylabel('Frequency')
plt.title('Histogram blockDeb')
# Display the plot
plt.show()


# Plotting a basic histogram
#plt.hist(parsed_data3, range=[0,1700], bins=1700, color='skyblue', edgecolor='black')
plt.hist(parsed_data3, color='skyblue', edgecolor='black')
# Adding labels and title
plt.xlabel('Crystal ID')
plt.ylabel('Frequency')
plt.title('Histogram crystal ID')
# Display the plot
plt.show()

# Plotting a basic histogram
#plt.hist(parsed_data4, range=[0,1700], bins=425, color='skyblue', edgecolor='black')
plt.hist(parsed_data4, color='skyblue', edgecolor='black')
#plt.yscale('log')
# Adding labels and title
plt.xlabel('Position X')
plt.ylabel('Frequency')
plt.title('Histogram Position X')
# Display the plot
plt.show()


# Plotting a basic histogram
#plt.hist(parsed_data4, range=[0,1700], bins=425, color='skyblue', edgecolor='black')
plt.hist(parsed_data5, color='skyblue', edgecolor='black')
#plt.yscale('log')
# Adding labels and title
plt.xlabel('Position Y')
plt.ylabel('Frequency')
plt.title('Histogram Position Y')
# Display the plot
plt.show()


# Plotting a basic histogram
#plt.hist(parsed_data4, range=[0,1700], bins=425, color='skyblue', edgecolor='black')
plt.hist(parsed_data6, color='skyblue', edgecolor='black')
#plt.yscale('log')
# Adding labels and title
plt.xlabel('Position Z')
plt.ylabel('Frequency')
plt.title('Histogram Position Z')
# Display the plot
plt.show()




# Plotting a basic histogram
#plt.hist2d(parsed_data3, parsed_data4, range=([0,1700],[0,1700]), bins=(170,85))
plt.hist2d(parsed_data4, parsed_data5)
plt.colorbar()
# Adding labels and title
plt.xlabel('Position X')
plt.ylabel('Position Y')
plt.title('Position X vs. Y')
# Display the plot
plt.show()


# Plotting a basic histogram
#plt.hist2d(parsed_data3, parsed_data4, range=([0,1700],[0,1700]), bins=(170,85))
plt.hist2d(parsed_data4, parsed_data6)
plt.colorbar()
# Adding labels and title
plt.xlabel('Position X')
plt.ylabel('Position Z')
plt.title('Position X vs. Z')
# Display the plot
plt.show()


# Plotting a basic histogram
#plt.hist2d(parsed_data3, parsed_data4, range=([0,1700],[0,1700]), bins=(170,85))
plt.hist2d(parsed_data5, parsed_data6)
plt.colorbar()
# Adding labels and title
plt.xlabel('Position Y')
plt.ylabel('Position Z')
plt.title('Position Y vs. Z') 
# Display the plot
plt.show()


# Plotting a basic plot
#plt.hist2d(parsed_data3, parsed_data4, range=([0,1700],[0,1700]), bins=(170,85))
plt.scatter(parsed_data4, parsed_data5)
# Adding labels and title
plt.xlabel('Position X')
plt.ylabel('Position Y')
plt.title('PLOT Position X vs. Y')
# Display the plot
plt.show()

# Plotting a basic plot
#plt.hist2d(parsed_data3, parsed_data4, range=([0,1700],[0,1700]), bins=(170,85))
plt.scatter(parsed_data5, parsed_data6)
# Adding labels and title
plt.xlabel('Position Y')
plt.ylabel('Position Z')
plt.title('PLOT Position Y vs. Z')
# Display the plot
plt.show()




# Plotting a basic plot
#plt.hist2d(parsed_data3, parsed_data4, range=([0,1700],[0,1700]), bins=(170,85))
plt.scatter(parsed_data3, parsed_data4)
# Adding labels and title
plt.xlabel('CrystalID')
plt.ylabel('Position X')
plt.title('PLOT CrystalID vs. X')
# Display the plot
plt.show()

# Plotting a basic plot
#plt.hist2d(parsed_data3, parsed_data4, range=([0,1700],[0,1700]), bins=(170,85))
plt.scatter(parsed_data3, parsed_data5)
# Adding labels and title
plt.xlabel('CrystalID')
plt.ylabel('Position Y')
plt.title('PLOT CrystalID vs. Y')
# Display the plot
plt.show()

# Plotting a basic plot
#plt.hist2d(parsed_data3, parsed_data4, range=([0,1700],[0,1700]), bins=(170,85))
plt.scatter(parsed_data3, parsed_data6)
# Adding labels and title
plt.xlabel('CrystalID')
plt.ylabel('Position Z')
plt.title('PLOT CrystalID vs. Z')
# Display the plot
plt.show()




# importing mplot3d toolkits, numpy and matplotlib
from mpl_toolkits import mplot3d

fig = plt.figure()

# syntax for 3-D projection
ax = plt.axes(projection ='3d')

# defining all 3 axis
z = parsed_data6
x = parsed_data4
y = parsed_data5
ax.scatter(x, y, z)

# plotting
ax.scatter3D(x, y, z, 'green')
ax.set_xlabel('$X$', fontsize=20)
ax.set_ylabel('$Y$', fontsize=20)
ax.set_zlabel('$Z$', fontsize=20)
ax.set_title('3D line plot geeks for geeks')
plt.show()














